
class Portfolio {

    constructor({ positions, applyFee }) {
        this.positions = positions
        this.applyFee = applyFee
        this.prices = this.positions.reduce((a, c) => { a[c.instrument] = c.price; return a }, {})
        this.transactionFeeRate = 0.0001
        this.transactionFeeMin = 2.5
        this.total = null
        this._update()
    }

    _changeCash(amount) {
        const position = this.positions.find(p => p.instrument === "USD")
        position.quantity += amount
    }

    _changePosition(instrument, amount) {
        let position = this.positions.find(p => p.instrument === instrument)
        if (position == null) {
            position = { instrument, price: 0, quantity: 0, value: 0 }
        }
        position.quantity += amount
    }

    _update() {
        this.positions.forEach(p => p.price = this.prices[p.instrument])
        this.positions.forEach(p => p.value = p.quantity * p.price)
        this.total = this.positions.reduce((a, c) => a + c.value, 0)
    }

    // ------------------------------------------------------------------------------------
    // PUBLIC
    // ------------------------------------------------------------------------------------
    setPrice(instrument, price) {
        this.prices[instrument] = price
        this._update()
    }

    order(instrument, quantity) {
        const value = this.prices[instrument] * quantity
        this._changeCash(-value)

        if (this.applyFee) {
            const fee = Math.max(value * this.transactionFeeRate, this.transactionFeeMin)
            this._changeCash(-fee)
        }

        this._changePosition(instrument, quantity)
        this._update()
    }

    getPosition(instrument) {
        const position = this.positions.find(p => p.instrument === instrument)
        const result = { instrument, price: 0, quantity: 0, value: 0 }
        if (position !== null) {
            result.price = position.price
            result.quantity = position.quantity
            result.value = position.value
        }
        Object.freeze(result) // ojbect copied and freezed it prevents change from outside 
        return result
    }

    toObject() {
        const result = {
            positions: this.positions,
            total: this.total
        }
        Object.freeze(result) // ojbect copied and freezed it prevents change from outside 
        return result
    }
}

module.exports = Portfolio